@@ -1,12 +1,12 @@
 # -*- coding: utf-8 -*-
-'''isort:skip_file'''
-__title__ = 'cpenv'
-__version__ = '0.5.28'
-__author__ = 'Dan Bradham'
-__email__ = 'danielbradham@gmail.com'
-__url__ = 'http://github.com/cpenv/cpenv'
-__license__ = 'MIT'
-__description__ = 'Cross-platform Python environment management.'
+"""isort:skip_file"""
+__title__ = "cpenv"
+__version__ = "0.5.34"
+__author__ = "Dan Bradham"
+__email__ = "danielbradham@gmail.com"
+__url__ = "http://github.com/cpenv/cpenv"
+__license__ = "MIT"
+__description__ = "Cross-platform Python environment management."
 
 
 # Local imports
@@ -24,4 +24,5 @@ from . import mappings
 
 # Initialize cpenv
 from .api import _init
+
 _init()
