@@ -7,6 +7,7 @@ from cpenv.cli import core
 
 def perform_self_version_check():
     from cpenv import _self_version_check
+
     is_latest, current, latest = _self_version_check.is_latest_version()
     if not is_latest:
         _self_version_check.warn_newer_version_available(current, latest)
@@ -18,5 +19,5 @@ def main():
     core.run(cli.CpenvCLI, sys.argv[1:])
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
