@@ -17,11 +17,11 @@ if is_py3:
     numeric_types = (int, float)
 
 
-platform = sys.platform.rstrip('1234567890').lower()
-if platform == 'darwin':  # Use mac instead of darwin
-    platform = 'mac'
-os.environ['CPENV_PLATFORM'] = platform
+platform = sys.platform.rstrip("1234567890").lower()
+if platform == "darwin":  # Use mac instead of darwin
+    platform = "mac"
+os.environ["CPENV_PLATFORM"] = platform
 
 
 # Leave osx in supported platforms for backwards compatability
-supported_platforms = ['win', 'linux', 'mac', 'osx']
+supported_platforms = ["win", "linux", "mac", "osx"]
