@@ -5,7 +5,7 @@ missing = object()
 
 
 class Environment(object):
-    '''An object that represents a list of requirements.
+    """An object that represents a list of requirements.
 
     Once you have activated some modules, you can save them as an Environment
     to be activated later. Think of them as an alias for activating a list
@@ -15,11 +15,11 @@ class Environment(object):
     all Repos. The Shotgun app tk-cpenv implements this functionality
     internally, it should be moved to cpenv at some point and the code related
     to Environments in tk-cpenv can be removed.
-    '''
+    """
 
     def __init__(self, name, data=None, path=None):
         self.name = name
-        self.data = data or {'name': name, 'requires': []}
+        self.data = data or {"name": name, "requires": []}
         self.path = path
 
     def get(self, key, default=missing):
@@ -33,4 +33,4 @@ class Environment(object):
 
     @property
     def requires(self):
-        return self.data['requires']
+        return self.data["requires"]
