@@ -19,9 +19,8 @@ except ImportError:
     from .vendor import certifi
 
 
-
 def get(url):
-    '''Make a get request.'''
+    """Make a get request."""
 
     context = ssl.create_default_context(cafile=ca_certs())
     response = urlopen(url, context=context)
@@ -29,13 +28,13 @@ def get(url):
 
 
 def json(response):
-    '''Get dict from json response.'''
+    """Get dict from json response."""
 
     return json_load(response.read().decode())
 
 
 def ca_certs():
-    '''Returns path to vendored certifi/cacert.pem.'''
+    """Returns path to vendored certifi/cacert.pem."""
 
     package = os.path.dirname(__file__)
-    return os.path.join(package, 'vendor', 'certifi', 'cacert.pem')
+    return os.path.join(package, "vendor", "certifi", "cacert.pem")
