@@ -11,24 +11,24 @@ from .reporter import get_reporter
 from .repos import LocalRepo
 
 __all__ = [
-    'ResolveError',
-    'Resolver',
-    'Activator',
-    'Copier',
-    'Localizer',
-    'module_resolvers',
-    'is_redirecting',
-    'redirect_to_modules',
-    'parse_redirect',
+    "ResolveError",
+    "Resolver",
+    "Activator",
+    "Copier",
+    "Localizer",
+    "module_resolvers",
+    "is_redirecting",
+    "redirect_to_modules",
+    "parse_redirect",
 ]
 
 
 class ResolveError(Exception):
-    '''Raised when a Resolver fairs to resolve a module or list of modules.'''
+    """Raised when a Resolver fairs to resolve a module or list of modules."""
 
 
 class Resolver(object):
-    '''Responsible for resolving ModuleSpecs from requirement strings.
+    """Responsible for resolving ModuleSpecs from requirement strings.
 
     Modules will be looked up sequentially in all registered Repos. By default
     this order is:
@@ -41,21 +41,21 @@ class Resolver(object):
     If there are still unresolved modules, fallback to the old algorithm
     for module lookups using the resolve functions in the
     cpenv.resolver.module_resolvers list.
-    '''
+    """
 
     def __init__(self, repos):
         self.repos = repos
         self.reporter = get_reporter()
 
     def resolve(self, requirements):
-        '''Given a list of requirement strings, resolve ModuleSpecs.
+        """Given a list of requirement strings, resolve ModuleSpecs.
 
         Returns:
             list of ModuleSpec objects
 
         Raises:
             ResolveError when a requirement can not be resolved.
-        '''
+        """
 
         self.reporter.start_resolve(requirements)
         unresolved = list(requirements)
@@ -70,7 +70,8 @@ class Resolver(object):
             #       possibly use the new resolvelib being developed by pypa
 
             match_gen = (
-                module_spec for repo in self.repos
+                module_spec
+                for repo in self.repos
                 for module_spec in repo.find(requirement)
             )
 
@@ -90,25 +91,23 @@ class Resolver(object):
         self.reporter.end_resolve(resolved, unresolved)
 
         if unresolved:
-            raise ResolveError(
-                'Could not resolve: ' + ' '.join(unresolved)
-            )
+            raise ResolveError("Could not resolve: " + " ".join(unresolved))
 
         return resolved
 
 
 class Activator(object):
-    '''Responsible for activating modules.'''
+    """Responsible for activating modules."""
 
     def __init__(self, localizer=None):
-        self.localizer = localizer or Localizer(to_repo='home')
+        self.localizer = localizer or Localizer(to_repo="home")
 
     def combine_modules(self, modules):
-        '''Combine a list of module's environments.'''
+        """Combine a list of module's environments."""
         return mappings.join_dicts(*[obj.environment for obj in modules])
 
     def activate(self, module_specs):
-        '''Activate a list of module specs.'''
+        """Activate a list of module specs."""
 
         modules = self.localizer.localize(module_specs)
         env = self.combine_modules(modules)
@@ -121,14 +120,15 @@ class Activator(object):
 
 
 class Copier(object):
-    '''Responsible for copying modules to a specific module.'''
+    """Responsible for copying modules to a specific module."""
 
     def __init__(self, to_repo):
         from .api import get_repo
+
         self.to_repo = get_repo(to_repo)
 
     def copy(self, module_specs, overwrite=False):
-        '''Given ModuleSpecs, copy them to this copiers to_repo.'''
+        """Given ModuleSpecs, copy them to this copiers to_repo."""
         from .api import get_cache_path
 
         copied = []
@@ -149,7 +149,7 @@ class Copier(object):
                 module = Module.from_spec(module_spec)
             else:
                 download_path = get_cache_path(
-                    'tmp',
+                    "tmp",
                     str(hash(module_spec.repo.name)),
                     module_spec.name,
                     module_spec.version.string,
@@ -167,7 +167,7 @@ class Copier(object):
             )
             copied.append(new_module_spec)
 
-        tmp = get_cache_path('tmp')
+        tmp = get_cache_path("tmp")
         if os.path.isdir(tmp):
             paths.rmtree(tmp)
 
@@ -178,35 +178,33 @@ class Copier(object):
 
 
 class Localizer(object):
-    '''Downloads modules from remote Repos to a specific LocalRepo.
+    """Downloads modules from remote Repos to a specific LocalRepo.
 
     This is similar to a copy operation, but skips all module_specs that are
     already in LocalRepos. If they are in LocalRepos then they are already
     available to be activated.
-    '''
+    """
 
-    def __init__(self, to_repo='home'):
+    def __init__(self, to_repo="home"):
         from .api import get_repo
 
         self.to_repo = get_repo(to_repo)
         self.reporter = get_reporter()
 
         if not isinstance(self.to_repo, LocalRepo):
-            raise ValueError(
-                'Localizer expected LocalRepo got %s' % type(to_repo)
-            )
+            raise ValueError("Localizer expected LocalRepo got %s" % type(to_repo))
 
     def localize(self, module_specs, overwrite=False):
-        '''Given ModuleSpecs, download them to this Localizers repo.'''
+        """Given ModuleSpecs, download them to this Localizers repo."""
 
         self.reporter.start_localize(module_specs)
         localized = []
         for module_spec in module_specs:
+            self.reporter.localize_module(module_spec, None)
 
             # Module is already local
             if isinstance(module_spec.repo, LocalRepo):
                 localized.append(Module(module_spec.path))
-                self.reporter.localize_module(module_spec, localized[-1])
                 continue
 
             # Module already exists in to_repo
@@ -216,18 +214,14 @@ class Localizer(object):
                 if is_exact_match(module_spec.qual_name, match):
                     already_exists = True
                     localized.append(Module(match.path))
-                    self.reporter.localize_module(module_spec, localized[-1])
                     break
 
             if already_exists and not overwrite:
-                self.reporter.localize_module(module_spec, localized[-1])
                 continue
 
             # Generate a new module path in to_repo
             if module_spec.version.string in module_spec.real_name:
-                new_module_path = self.to_repo.relative_path(
-                    module_spec.real_name
-                )
+                new_module_path = self.to_repo.relative_path(module_spec.real_name)
             else:
                 new_module_path = self.to_repo.relative_path(
                     module_spec.name,
@@ -240,7 +234,6 @@ class Localizer(object):
                 overwrite=overwrite,
             )
             localized.append(module)
-            self.reporter.localize_module(module_spec, localized[-1])
 
         self.reporter.end_localize(localized)
 
@@ -251,10 +244,10 @@ class Localizer(object):
 
 
 def old_resolve_algorithm(resolver, paths):
-    '''Deprecated: Pre-0.5.0 resolution algorithm.
+    """Deprecated: Pre-0.5.0 resolution algorithm.
 
     Included for backwards compatability.
-    '''
+    """
 
     modules = []
     for path in list(paths):
@@ -278,9 +271,9 @@ def old_resolve_algorithm(resolver, paths):
 
 
 def system_path_resolver(resolver, path):
-    '''Checks if path is already a :class:`Module` object'''
+    """Checks if path is already a :class:`Module` object"""
 
-    is_system_path = ('/' in path or '\\' in path or path.startswith('.'))
+    is_system_path = "/" in path or "\\" in path or path.startswith(".")
     if is_system_path:
         mod_path = paths.normalize(path)
         if is_module(mod_path):
@@ -292,9 +285,9 @@ def system_path_resolver(resolver, path):
 
 
 def redirect_resolver(resolver, path):
-    '''Resolves environment from .cpenv file...recursively walks up the tree
+    """Resolves environment from .cpenv file...recursively walks up the tree
     in attempt to find a .cpenv file
-    '''
+    """
 
     if not os.path.exists(path):
         raise ResolveError
@@ -304,9 +297,7 @@ def redirect_resolver(resolver, path):
 
     for root, _, _ in paths.walk_up(path):
         if is_redirecting(root):
-            env_paths = redirect_to_modules(
-                paths.normalize(root, '.cpenv')
-            )
+            env_paths = redirect_to_modules(paths.normalize(root, ".cpenv"))
             r = Resolver(resolver.repos)
             return r.resolve(env_paths)
 
@@ -320,25 +311,25 @@ module_resolvers = [
 
 
 def is_redirecting(path):
-    '''Returns True if path contains a .cpenv file'''
+    """Returns True if path contains a .cpenv file"""
 
-    candidate = paths.normalize(path, '.cpenv')
+    candidate = paths.normalize(path, ".cpenv")
     return os.path.exists(candidate) and os.path.isfile(candidate)
 
 
 def redirect_to_modules(path):
-    '''Get environment path from redirect file'''
+    """Get environment path from redirect file"""
 
-    with open(path, 'r') as f:
+    with open(path, "r") as f:
         data = f.read()
 
     return parse_redirect(data)
 
 
 def parse_redirect(data):
-    '''Parses a redirect string - data of a .cpenv file'''
+    """Parses a redirect string - data of a .cpenv file"""
 
-    lines = [line for line in data.split('\n') if line.strip()]
+    lines = [line for line in data.split("\n") if line.strip()]
     if len(lines) == 1:
         return shlex.split(lines[0])
     else:
