@@ -9,45 +9,39 @@ from functools import total_ordering
 from . import compat
 
 __all__ = [
-    'ParseError',
-    'Version',
-    'parse_version',
-    'default_version',
+    "ParseError",
+    "Version",
+    "parse_version",
+    "default_version",
 ]
 
 
 # Strange nuke version pattern
-nuke_version_pattern = (
-    r'(?P<major>\d+)'
-    r'\.(?P<minor>\d+)'
-    r'(?:v)'
-    r'(?P<patch>\d+)$'
-)
+nuke_version_pattern = r"(?P<major>\d+)" r"\.(?P<minor>\d+)" r"(?:v)" r"(?P<patch>\d+)$"
 
 # Version pattern with 4 digits
 four_version_pattern = (
-    r'(?:v)?'
-    r'(?P<major>\d+)'
-    r'\.(?P<minor>\d+)'
-    r'\.(?P<revision>\d+)'
-    r'\.(?P<build>\d+)'
-    r'(?:-(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$'
+    r"(?:v)?"
+    r"(?P<major>\d+)"
+    r"\.(?P<minor>\d+)"
+    r"\.(?P<revision>\d+)"
+    r"\.(?P<build>\d+)"
+    r"(?:-(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"
 )
 
 # Modified regex from semver.org - works with calver as well
 semver_version_pattern = (
-    r'(?:v)?'
-    r'(?P<major>\d+)'
-    r'\.(?P<minor>\d+)'
-    r'\.(?P<patch>\d+)'
-    r'(?:-(?P<prerelease>(?:\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)'
-    r'(?:\.(?:\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?'
-    r'(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$'
+    r"(?:v)?"
+    r"(?P<major>\d+)"
+    r"\.(?P<minor>\d+)"
+    r"\.(?P<patch>\d+)"
+    r"(?:-(?P<prerelease>(?:\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*)"
+    r"(?:\.(?:\d*|\d*[a-zA-Z-][0-9a-zA-Z-]*))*))?"
+    r"(?:\+(?P<buildmetadata>[0-9a-zA-Z-]+(?:\.[0-9a-zA-Z-]+)*))?$"
 )
-simplever_pattern = r'(?:v)?(?P<version>(\d+\.?)+)$'
+simplever_pattern = r"(?:v)?(?P<version>(\d+\.?)+)$"
 VersionBase = namedtuple(
-    'Version',
-    ['major', 'minor', 'patch', 'prerelease', 'buildmetadata', 'string']
+    "Version", ["major", "minor", "patch", "prerelease", "buildmetadata", "string"]
 )
 
 
@@ -55,11 +49,11 @@ VersionBase = namedtuple(
 class Version(VersionBase):
 
     _defaults = {
-        'major': 0,
-        'minor': 0,
-        'patch': 0,
-        'prerelease': None,
-        'buildmetadata': None
+        "major": 0,
+        "minor": 0,
+        "patch": 0,
+        "prerelease": None,
+        "buildmetadata": None,
     }
 
     def __str__(self):
@@ -69,7 +63,7 @@ class Version(VersionBase):
         return super(Version, self).__hash__()
 
     def _comparable_value(self, value, other):
-        '''Return a value that will be comparable with other.'''
+        """Return a value that will be comparable with other."""
 
         types_match = type(value) is type(other)
         if types_match or isinstance(value, compat.numeric_types):
@@ -79,20 +73,20 @@ class Version(VersionBase):
             return value
 
         if value is None and isinstance(other, compat.string_types):
-            return 'zzzzzzzz'
+            return "zzzzzzzz"
 
         if isinstance(other, compat.numeric_types):
-            return -float('inf')
+            return -float("inf")
 
         return value
 
     def _comparable(self, other):
-        '''Generate a comparison key for a Version object.
+        """Generate a comparison key for a Version object.
 
         Coerces types of prerelease and buildmeta to ensure that the Version
         objects are comparable. This is required because Python 3 now raises
         a TypeError when attempting to compare str and int.
-        '''
+        """
         return (
             self.major,
             self.minor,
@@ -103,23 +97,23 @@ class Version(VersionBase):
 
     def __lt__(self, other):
         if not isinstance(other, Version):
-            raise ValueError('Can only compare two Version objects.')
+            raise ValueError("Can only compare two Version objects.")
 
         return self._comparable(other) < other._comparable(self)
 
     def __eq__(self, other):
         if not isinstance(other, Version):
-            raise ValueError('Can only compare two Version objects.')
+            raise ValueError("Can only compare two Version objects.")
 
         return tuple(other) == tuple(self)
 
 
 class ParseError(Exception):
-    '''Raised when a parse method fails.'''
+    """Raised when a parse method fails."""
 
 
 def parse_version(string):
-    '''Parse and return a Version from the provided string.
+    """Parse and return a Version from the provided string.
 
     Supports:
       - semver / calver
@@ -133,28 +127,28 @@ def parse_version(string):
 
     Raises:
         ParseError when a version can not be parsed.
-    '''
+    """
     # Parse weird nuke versioning
     match = re.search(nuke_version_pattern, string)
     if match:
         return Version(
-            major=int(match.group('major')),
-            minor=int(match.group('minor')),
-            patch=int(match.group('patch')),
+            major=int(match.group("major")),
+            minor=int(match.group("minor")),
+            patch=int(match.group("patch")),
             prerelease=None,
             buildmetadata=None,
-            string=match.group(0)
+            string=match.group(0),
         )
 
     # Parse four_version - four digit version
     match = re.search(four_version_pattern, string)
     if match:
         return Version(
-            major=int(match.group('major')),
-            minor=int(match.group('minor')),
-            patch=int(match.group('revision')),
-            prerelease=int(match.group('build')),
-            buildmetadata=match.group('buildmetadata'),
+            major=int(match.group("major")),
+            minor=int(match.group("minor")),
+            patch=int(match.group("revision")),
+            prerelease=int(match.group("build")),
+            buildmetadata=match.group("buildmetadata"),
             string=match.group(0),
         )
 
@@ -162,11 +156,11 @@ def parse_version(string):
     match = re.search(semver_version_pattern, string)
     if match:
         return Version(
-            major=int(match.group('major')),
-            minor=int(match.group('minor')),
-            patch=int(match.group('patch')),
-            prerelease=match.group('prerelease'),
-            buildmetadata=match.group('buildmetadata'),
+            major=int(match.group("major")),
+            minor=int(match.group("minor")),
+            patch=int(match.group("patch")),
+            prerelease=match.group("prerelease"),
+            buildmetadata=match.group("buildmetadata"),
             string=match.group(0),
         )
 
@@ -174,13 +168,13 @@ def parse_version(string):
     match = re.search(simplever_pattern, string)
     if match:
         kwargs = dict(Version._defaults)
-        kwargs['string'] = match.group(0)
-        version_parts = match.group('version').split('.')
-        for part, part_name in zip(version_parts, ['major', 'minor', 'patch']):
+        kwargs["string"] = match.group(0)
+        version_parts = match.group("version").split(".")
+        for part, part_name in zip(version_parts, ["major", "minor", "patch"]):
             kwargs[part_name] = int(part)
         return Version(**kwargs)
 
-    raise ParseError('Could not parse version from %s' % string)
+    raise ParseError("Could not parse version from %s" % string)
 
 
 def default_version():
@@ -190,5 +184,5 @@ def default_version():
         patch=0,
         prerelease=None,
         buildmetadata=None,
-        string='0.1.0',
+        string="0.1.0",
     )
